#!/usr/bin/perl -w
#####################################################
# gdiptinydns.pl
#
# This script scans the database and creates  data
# file for tinydns. Please make sure that other
# required dns entries are written to this file
# before this script is run.
#
# See COPYING for licensing information.
#
# Originally written by:
#
#   Creighton MacDonnell
#   http://macdonnell.ca/
#
# Adapted for use with tinydns by:
#
#   Roel van Meer
#   rolek@lsof.org
#
# This script was not tested since the author
# does not use tinydns. Based on a read of the 
# documentation for the tinydns-data program, there
# is a problem with the definition of the MX
# records in this script. The "&" character is for
# name servers not mail exchangers according to that
# documentation. Moreover there does not appear to be
# a way to define a mail exchanger without providing
# an IP address, or a mail exchanger that is outside
# the current domain.
#
# If you make this script work with mail exchangers,
# please send a copy to the mailing list.
#
#####################################################

# Perl modules
use strict;
use Getopt::Std;

# global variables
use vars qw($conf $gnudipdir);

# locate ourselves
use FindBin;
BEGIN {
  $gnudipdir = '';
  if ($FindBin::Bin =~ /(.*)\/.+?/) {
    $gnudipdir = $1;
  }
}
use lib "$gnudipdir/lib";

# GnuDIP common subroutines
use gdiplib;
use dbusers;

# process command line
sub usage {
  print STDERR <<"EOQ";
usage: gdiptinydns.pl [ -h | [ [-o | -a] outfile ] [domain] ]
usage: Scans the database and generates tinydns input to create domain
usage: file. Scans for all domains unless "domain" is specified.
usage: -h: Print this usage message.
usage: -o: Specify file to write output to.
usage: -a: Specify file to append output to.
EOQ
}
use vars qw( $opt_h $opt_o $opt_a);
if (!getopts('ho:a:')) {
  usage();
  exit 1;
}
if (@ARGV gt 1) {
  usage();
  exit 1;
}
if ($opt_h) {
  usage();
  exit;
}

# redirect output?
if ($opt_o) {
  close(STDOUT);
  open(STDOUT, ">$opt_o");
}
if ($opt_a) {
  close(STDOUT);
  open(STDOUT, ">>$opt_a");
}

# domain specified?
my $domain = '';
if (@ARGV eq 1) {
  $domain = shift;
}

# get preferences from config file
$conf = getconf();

# scan users
my $sth;
if ($domain) {
  $sth = getusersdomain($domain);
} else {
  $sth = getusers();
}
my $recs = 0;
my $users = 0;
while (my $uinfo = getuserseach($sth)) {

  if ($$uinfo{'domain'}) {

    my $TTL = 0;
    $TTL = $$conf{'TTL'} if $$conf{'TTL'};
    $TTL = $$conf{"TTL.$$uinfo{'domain'}"}
        if $$conf{"TTL.$$uinfo{'domain'}"};

    my $marker = '';

    if ($$uinfo{'currentip'} ne '0.0.0.0') {
      print "+$$uinfo{'username'}.$$uinfo{'domain'}:$$uinfo{'currentip'}:$TTL\n";
      $recs++;
      $marker = 1;

      if ($$uinfo{'MXbackup'} eq 'YES') {
        # this appears to be wrong!
        print "&$$uinfo{'username'}.$$uinfo{'domain'}:$$uinfo{'currentip'}::200:$TTL\n";
        $recs++;
      }

      if ($$uinfo{'wildcard'} eq 'YES') {
        print "+*.$$uinfo{'username'}.$$uinfo{'domain'}:$$uinfo{'currentip'}:$TTL\n";
        $recs++;
      }
    }

    if ($$uinfo{'MXvalue'}) {
      # this appears to be wrong!
      print "&$$uinfo{'username'}.$$uinfo{'domain'}:$$uinfo{'MXvalue'}::100:$TTL\n";
      $recs++;
      $marker = 1;
    }

    if ($marker) {
      $users++;
    }
  }
}

# final message
writelog("$recs tinydns records written for $users users");

exit;

#####################################################

# write to the "log"
sub writelog {
  my $logstr = shift;
  print STDERR "$logstr\n";
}

# called for database error
sub dberror {
  exit;
}

